<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/2000/svg">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	<xsl:template match="/">
		<svg xmlns="http://www.w3.org/2000/svg" xmlns:be="http://www.complinguist.org/syntax/tree/baseElements" xmlns:stb="http://www.complinguist.org/syntax/tree" xmlns:xlink="http://www.w3.org/1999/xlink" onload="init(evt)">
			<title>Syntax Tree</title>
			<script type="text/ecmascript">
			/* SyntaxTree class */

			//SyntaxTree constructor - - - - - - - - - - - - - - - - -
			function SyntaxTree(rootNode){
				//properties
				this.root = rootNode;
				this.leafNodes = Array();
				this.areDropped = false;

				//declare methods
				this.draw = SyntaxTree_draw;
				this._draw = SyntaxTree_draw_; //private
				this.dropLeaves = SyntaxTree_dropLeaves;
				this.retractLeaves = SyntaxTree_retractLeaves;
				this.toggleLeafDrop = function(){
					if(this.areDropped)
						this.retractLeaves();
					else
						this.dropLeaves();
				};

				//CSS analogous properties (all in pixels)
				this.fontSize = 20;
				this.wordSpacing = 10;
				this.marginLeft = 10;
				this.marginTop = 10;
				this.width = 0;   //read only
				this.height = 0;  //read only
				this.branchHeight = this.fontSize;
				this.branchPaddingTop = this.fontSize/4;
				this.branchPaddingBottom = 0;

				var thisRef = this; //so the following can reference "this"

				//define event handlers - - - - - - - - -
				function handleTextClick(evt){
					var classValue = evt.target.getAttribute('class');
					//uncollapse node
					if(classValue.match(/collapse/)){
						evt.target.setAttribute('class', classValue.replace(/ ?collapse( )?/, "$1"));
						thisRef.draw();
					}
					//collapse node
					else {
						evt.target.setAttribute('class', classValue + ' collapse');
						thisRef.draw();
					}
				}

				//assign event handlers - - - - - - - - -
				var i;
				var allTextNodes = rootNode.getElementsByTagName('text');
				for(i = 0; i &lt; allTextNodes.length; i++){
					if(allTextNodes.item(i).getAttribute('class').match(/phrasalNode/)){
						allTextNodes.item(i).addEventListener('click', handleTextClick, false);
					}
				}
			}


			//SyntaxTree methods

			//draws the tree
			function SyntaxTree_draw(){
				this.width = 0;
				this.height = 0;
				this.leafNodes = Array();
				this._draw(this.root, null, false, this.marginLeft, this.marginTop + this.fontSize);
				if(this.areDropped)
					this.dropLeaves();
			}

			//recursive draw method that is called for the draw method
			function SyntaxTree_draw_(containerNode, parentLabel, isNullParent, shiftLeft, shiftTop){
				//variables needed in this call
				var childrenWidth = 0;
				var childrenLabels = Array();
				var nodeName;
				var nodeClass;
				var nodeLabel;
				var nodeBranch;
				var nodeTriangle;
				var nodeLabelWidth;
				var isNull = false;
				var isCollapsed = false;
				var leafY;

				//get and handle children ============================
				var children = containerNode.childNodes();
				for(var i = 0; i &lt; children.length; i++){
					nodeName = children.item(i).nodeName();
					//the node label and type
					if(nodeName == 'text'){
						nodeLabel = children.item(i);
						nodeLabel.setAttribute('display', 'block');
						classNames = nodeLabel.getAttribute('class').replace(/^\s+|\s+$/g, '').split(/\s+/);

						for(var j = 0; j &lt; classNames.length; j++){
							if(classNames[j] == 'collapse')
								isCollapsed = true;
							else
								nodeClass = classNames[j];
						}

						//phrasal or lexical node
						if(nodeClass == 'phrasalNode' || nodeClass == 'lexicalNode'){
							if(nodeLabel.firstChild())
								nodeLabel.firstChild().nodeValue = nodeLabel.firstChild().nodeValue.replace(/_/g, "'");
						}

						//determine the width of the label
						if(!nodeLabel.firstChild() || !nodeLabel.firstChild().nodeValue || nodeLabel.firstChild().nodeValue.match(/^\s*$/) ){
							nodeLabelWidth = this.wordSpacing;
							isNull = true;
						}
						else {
							nodeLabelWidth = this.wordSpacing + nodeLabel.getComputedTextLength();
							isNull = false;
						}
					}
					//connecting branch
					else if(nodeName == "line"){
						nodeBranch = children.item(i);
					}
					//collpase triangle
					else if(nodeName == "path"){
						nodeTriangle = children.item(i);
					}
					//children nodes
					else if(nodeName == "g"){
						//collapsed node: draw child labels directly underneath the triangle
						if(isCollapsed &amp;&amp; nodeTriangle){
							var j;

							//hide all descendant branches
							var descendantBranches = children.item(i).getElementsByTagName('line');
							for(j = 0; j &lt; descendantBranches.length; j++)
								descendantBranches.item(j).setAttribute('display', 'none');

							//hide all descendant triangles
							var descendantTriangles = children.item(i).getElementsByTagName('path');
							for(j = 0; j &lt; descendantTriangles.length; j++)
								descendantTriangles.item(j).setAttribute('display', 'none');

							//gather and display all leaf nodes
							var descendantLabels = children.item(i).getElementsByTagName('text');
							var descendantLabel = null;
							var descendantWidth = 0;

							//get the y coordinate of the labels under the triangle
							var y = shiftTop + this.branchHeight;
							if(isNull) //under null, triangle (branch) is shifted up
								y += -this.branchPaddingBottom - this.branchPaddingTop;
							else
								y += this.fontSize;

							//position and process each leaf
							for(j = 0; j &lt; descendantLabels.length; j++){
								var descendantLabel = descendantLabels.item(j);
								if(!descendantLabel.getAttribute('class').match(/\b(lexicalData|trace|PRO|pro)\b/)){
									descendantLabel.setAttribute('display', 'none');
									continue;
								}
								descendantLabel.setAttribute('display', 'block');
								descendantWidth = descendantLabel.getComputedTextLength() + this.wordSpacing;
								descendantLabel.setAttribute('x', (shiftLeft + childrenWidth + descendantWidth/2) + 'px');
								descendantLabel.setAttribute('y', y + 'px');
								childrenWidth += descendantWidth;
								childrenLabels.push(descendantLabel);

								//set tree height if we are higher than the current height
								if(shiftTop + this.branchHeight + this.fontSize &gt; this.height)
									this.height = shiftTop + this.branchHeight + this.fontSize;
							}
							leafY = y;
						}
						//traverse child trees
						else {
							var returned = this._draw(children.item(i), nodeLabel, isNull, shiftLeft + childrenWidth, shiftTop + (isNull ? (nodeClass == 'phrasalNode' ? this.branchHeight : 0) : this.branchHeight + this.fontSize));
							childrenLabels.push(returned[0]);
							childrenWidth += returned[1];
						}
					}
				}

				//draw label, children, and branches ==================
				if(!childrenWidth) //there are no children; this is the branch end
					childrenWidth = nodeLabelWidth;
				if(nodeLabel == null)
					throw Error("Error: Every child must have a label (every &lt;g&gt; must contain a &lt;text&gt;).");

				//position label
				var thisY = shiftTop;
				var thisX;
				if(!isCollapsed &amp;&amp; childrenLabels.length){
					var firstChild = parseFloat(childrenLabels[0].label.getAttribute('x'));
					var lastChild = parseFloat(childrenLabels[childrenLabels.length-1].label.getAttribute('x'));
					thisX = firstChild + (lastChild - firstChild)/2;
				}
				else {
					thisX = shiftLeft + childrenWidth/2;
				}
				if(thisY &gt; this.height) this.height = thisY;
				if(shiftLeft + childrenWidth &gt; this.width) this.width = shiftLeft + childrenWidth;
				nodeLabel.setAttribute('y', thisY + 'px'); //possible: don't print lexical data branches option
				nodeLabel.setAttribute('x', thisX + 'px');
				nodeLabel.setAttribute('display', 'block');

				//draw triangle
				if(isCollapsed &amp;&amp; nodeTriangle){
					nodeTriangle.setAttribute('display', 'block');
					var topX = thisX;
					var topY = thisY + (isNull ? -this.fontSize - this.branchPaddingBottom : this.branchPaddingTop);
					nodeTriangle.setAttribute('d', 'M' + (topX + " " + topY) + "L" + (topX - childrenWidth/2 + this.wordSpacing/2) + ' ' + (topY + this.branchHeight - this.branchPaddingTop - this.branchPaddingBottom) + ' l' + (childrenWidth - this.wordSpacing) + ' 0 z');
				}
				//connect branches from child labels to parent label
				else {
					leafY = thisY;
					if(nodeTriangle)
						nodeTriangle.setAttribute('display', 'none');
					for(var i = 0; i &lt; childrenLabels.length; i++){
						childrenLabels[i].branch.setAttribute('x1', thisX + 'px');
						childrenLabels[i].branch.setAttribute('y1', (thisY + (isNull ? -this.fontSize : this.branchPaddingTop)) + 'px');
					}
				}

				//below root: anchor one end of the branch to the label
				if(nodeBranch){
					nodeBranch.setAttribute('x2', nodeLabel.getAttribute('x'));
					nodeBranch.setAttribute('y2', (parseFloat(nodeLabel.getAttribute('y')) - this.fontSize - this.branchPaddingBottom) + 'px');
					nodeBranch.setAttribute('display', 'block');
					//add regular node to leafNodes
					if(!childrenLabels.length &amp;&amp; !isNull){
						this.leafNodes.push({
							isCollapsed:false,
							label:nodeLabel,
							branch:nodeBranch,
							parentLabel:parentLabel,
							yOrigin:leafY,
							width:nodeLabelWidth,
							isNullParent:isNullParent
						});
					}
					//add collapsed node to leafNodes
					else if(isCollapsed)
						this.leafNodes.push({
							isCollapsed:true,
							label:childrenLabels,
							branch:nodeTriangle,
							parentLabel:nodeLabel,
							yOrigin:leafY,
							width:childrenWidth,
							isNullParent:isNull
					});
				}
				return Array({label:nodeLabel, branch:nodeBranch}, childrenWidth);
			}

			//extends the lexicalData nodes (the words) to the bottom of the tree
			function SyntaxTree_dropLeaves(){
				this.areDropped = true;
				for(var i = 0; i &lt; this.leafNodes.length; i++){
					//extend collapsed node
					if(this.leafNodes[i].isCollapsed){
						for(var j = 0; j &lt; this.leafNodes[i].label.length; j++)
							this.leafNodes[i].label[j].setAttribute('y', this.height + 'px');

						//stretch triangle
						var topX = parseFloat(this.leafNodes[i].parentLabel.getAttribute('x'));
						var topY = parseFloat(this.leafNodes[i].parentLabel.getAttribute('y')) + (this.leafNodes[i].isNullParent ? -this.fontSize - this.branchPaddingBottom : this.branchPaddingTop);
						var bottomY = this.height -this.fontSize - this.branchPaddingBottom;
						this.leafNodes[i].branch.setAttribute('d', 'M' + (topX + " " + topY) + "L" + (topX - this.leafNodes[i].width/2 + this.wordSpacing/2) + ' ' + (bottomY) + ' l' + (this.leafNodes[i].width - this.wordSpacing) + ' 0 z');
					}
					//extend regular node
					else {
						this.leafNodes[i].label.setAttribute('y', this.height + 'px');
						this.leafNodes[i].branch.setAttribute('y2', (this.height - this.fontSize - this.branchPaddingBottom) + 'px');
					}
				}
			}

			//puts the lexicalData nodes back to their original locations
			function SyntaxTree_retractLeaves(){
				this.areDropped = false;
				for(var i = 0; i &lt; this.leafNodes.length; i++){
					var labelY = this.leafNodes[i].yOrigin;

					//retract collapsed node
					if(this.leafNodes[i].isCollapsed){
						//var totalWidth = 0;
						for(var j = 0; j &lt; this.leafNodes[i].label.length; j++)
							this.leafNodes[i].label[j].setAttribute('y', labelY + 'px');

						//unstretch triangle
						var topX = parseFloat(this.leafNodes[i].parentLabel.getAttribute('x'));
						var topY = parseFloat(this.leafNodes[i].parentLabel.getAttribute('y')) + (this.leafNodes[i].isNullParent ? -this.fontSize - this.branchPaddingBottom : this.branchPaddingTop);
						this.leafNodes[i].branch.setAttribute('d', 'M' + (topX + " " + topY) + "L" + (topX - this.leafNodes[i].width/2 + this.wordSpacing/2) + ' ' + (topY + this.branchHeight - this.branchPaddingTop - this.branchPaddingBottom) + ' l' + (this.leafNodes[i].width - this.wordSpacing) + ' 0 z');
					}
					//retract regular node
					else {
						this.leafNodes[i].label.setAttribute('y', labelY + 'px');
						this.leafNodes[i].branch.setAttribute('y2', labelY - this.fontSize - this.branchPaddingBottom + 'px');
					}
				}
			}

			</script>
			<script type="text/ecmascript">

			// Event Handlers

			var tree;
			function init(evt){
				tree = new SyntaxTree(document.getElementById('tree'));
				tree.marginLeft = 5;
				tree.marginTop = 5;
				tree.fontSize = 20; //due to bug in Adobe SVG viewer, must be the same as that in the stylesheet
				tree.wordSpacing = 25;
				tree.branchHeight = 20;
				tree.branchPaddingTop = 5;
				tree.branchPaddingBottom = 0;
				tree.draw();
				tree.dropLeaves();
			}
			</script>

			<g id="tree">
				<text/>
				<xsl:for-each select="*">
					<xsl:call-template name="pn"/>
				</xsl:for-each>
			</g>
		</svg>
	</xsl:template>
	<xsl:template name="pn">
		<xsl:choose>
			<xsl:when test="not(text())">
				<g>
					<!--<text class="phrasalNode">-->
					<text style="text-anchor:middle;font-family:'Lucida Console Unicode', Arial, Helvetica, Serif;fill:blue" class="phrasalNode">
						<xsl:value-of select="name()"/>
					</text>
					<path style="stroke:black;stroke-width:1px;fill:none" class="triangle"/>
					<!--<line class="branchLine"/>-->
					<line style="stroke:black" class="branchLine"/>
					<xsl:for-each select="*">
						<xsl:call-template name="pn"/>
					</xsl:for-each>
				</g>
			</xsl:when>
			<xsl:otherwise>
				<g>
					<!--<text class="lexicalNode">-->
					<text style="text-anchor:middle;font-family:'Lucida Console Unicode', Arial, Helvetica, Serif" class="lexicalNode">
						<xsl:value-of select="name()"/>
					</text>
					<!--<line class="branchLine"/>-->
					<line style="stroke:black" class="branchLine"/>
					<xsl:apply-templates/>
				</g>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="text()">
		<g>
			<!--<text class="lexicalData">-->
			<text style="text-anchor:middle;font-family:'Lucida Console Unicode', Arial, Helvetica, Serif;fill:red" class="lexicalData">
				<xsl:value-of select="."/>
			</text>
			<!--<line class="branchLine"/>-->
			<line style="stroke:black" class="branchLine"/>
		</g>
	</xsl:template>
</xsl:stylesheet>
